﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Observer
{

    class Program
    {
        static void Main()
        {
            DateTime now = DateTime.Now;

            //출발 시간과 목적지가 있는  새로운 비행편을 만든다

            CarrierSchedule jetBlue = new CarrierSchedule("JetBlue", now);
            jetBlue.Attach(new AirTrafficControl("Boston"));
            jetBlue.Attach(new AirTrafficControl("Seattle"));
            //ATC 클래스들은 출발 시간의 연기를 통지할 것이다
            jetBlue.DepartureDateTime = now.AddHours(1.25);// 날씨 때문에 연기
            jetBlue.DepartureDateTime = now.AddHours(1.75); //날씨가 더 악화됨
            jetBlue.DepartureDateTime = now.AddHours(0.5); //보안 떄문에 연기
            jetBlue.DepartureDateTime = now.AddHours(0.75); //시애틀 출발 시간

            // 승객 대기
            Console.Read();
        }
    }
    // 비행 스케줄 요청을 가로채기 위한 제네릭 델리게이트 타입 선언
    public delegate void ChangeEventhandler<T, U>(T sender, U eventArgs);

    // 실행에 맞는 이벤트 인수를 설정

    public class ChangeEventArgs : EventArgs
    {
        public string Airline { get; set; }
        public string DepartureAirport { get; set; }
        public string ArrivalAirport { get; set; }
        public DateTime DepartureDateTime { get; set; }

        public ChangeEventArgs(string name, string outAirport, string inAirport, DateTime leaves)
        {
            this.Airline = name;
            this.DepartureAirport = outAirport;
            this.ArrivalAirport = inAirport;
            this.DepartureDateTime = leaves;
        }


    }  // 주제 : Air Traffic Control 센터에서 관찰하게 되는 대상이다
    abstract class AirlineSchedule
    {
        public string Name { get; set; }
        public string DepartureAirport { get; set; }
        public string ArrivalAirport { get; set; }
        private DateTime departuredateTime;

        public AirlineSchedule(string ariline, string outAirport, string inAiport, DateTime leaves)
        {
            this.Name = ariline;
            this.DepartureAirport = outAirport;
            this.ArrivalAirport = inAiport;
            this.departuredateTime = leaves;
        }

        // 이벤트

        public event ChangeEventhandler<AirlineSchedule, ChangeEventArgs> Change;

        // Change 이벤트를 일으킨다
        public virtual void OnChange(ChangeEventArgs e)
        {
            if (Change != null)
            {
                Change(this, e);
            }
        }

        // 관찰자(observer)들을 실제로 추가하는 부분
        public void Attach(AirTrafficControl airTrafficControl)
        {
            Change += new ChangeEventhandler<AirlineSchedule, ChangeEventArgs>(airTrafficControl.Update);
        }

        public void Detach(AirTrafficControl airTrafficControl)
        {
            Change -= new ChangeEventhandler<AirlineSchedule, ChangeEventArgs>(airTrafficControl.Update);
        }

        public DateTime DepartureDateTime
        {
            get { return departuredateTime; }
            set
            {
                departuredateTime = value;
                OnChange(new ChangeEventArgs(this.Name, this.DepartureAirport, this.ArrivalAirport, this.departuredateTime));
                Console.WriteLine("<br/>");
            }
        }

    }


    // 실제 주제(concrete subect)
    class CarrierSchedule : AirlineSchedule
    {
        // 제시와 알렉스는 실제 한 곳으로만 비행할 곳을 한다
        public CarrierSchedule(string name, DateTime departing)
            : base(name, "Boston", "Seattle", departing)
        {

        }

    }

    // 관찰자
    interface IATC
    {
        void Update(AirlineSchedule sender, ChangeEventArgs e);
    }
    // 실제 관찰자
    class AirTrafficControl : IATC
    {
        public string Name { get; set; }

        // 생성자
        public AirTrafficControl(string name)
        {
            this.Name = name;
        }
        public void Update(AirlineSchedule sender, ChangeEventArgs e)
        {
            Console.WriteLine("{0} Air Traffic Control Notified :\n {1}'s flight 497 from {2} " +
                "to {3} new departure time : {4:hh:mmtt}", Name, e.ArrivalAirport, e.DepartureAirport, e.ArrivalAirport, e.DepartureDateTime);
            Console.WriteLine("------------------");
        }
        public CarrierSchedule CarrierSchedule { get; set; }
    }

}



